@@ -4,11 +4,13 @@
 
 ## From Version 0.9.12 the master repository for _dnslib_ has been moved to GitHub (https://github.com/paulc/dnslib). Please update any links to the original BitBucket repository as this will no longer be maintained.
 
+## Release 0.9.24 (2024-01-02) will be the last release supporting Python 2.7 and Python <3.7. Supporting old Python versions is increasingly painful and holds back adoption of new features so it's probably time to move on. 
+
+A library to encode/decode DNS wire-format packets supporting both
 
 dnslib
 ------
 
-A library to encode/decode DNS wire-format packets supporting both 
 Python 2.7 and Python 3.2+.
 
 The library provides:
@@ -16,8 +18,8 @@ The library provides:
  * Support for encoding/decoding DNS packets between wire format,
    python objects, and Zone/DiG textual representation (dnslib.dns)
 
- * A server framework allowing the simple creation of custom DNS 
-   resolvers (dnslib.server) and a number of example servers 
+ * A server framework allowing the simple creation of custom DNS
+   resolvers (dnslib.server) and a number of example servers
    created using this framework
 
  * A number of utilities for testing (dnslib.client, dnslib.proxy,
@@ -26,11 +28,11 @@ The library provides:
 Python 3 support was added in Version 0.9.0 which represented a fairly
 major update to the library - the key changes include:
 
- * Python 2.7/3.2+ support (the last version supporting Python 2.6 
+ * Python 2.7/3.2+ support (the last version supporting Python 2.6
    or earlier was version 0.8.3)
 
- * The 'Bimap' interface was changed significantly to explicitly 
-   split forward (value->text) lookups via __getitem__ and 
+ * The 'Bimap' interface was changed significantly to explicitly
+   split forward (value->text) lookups via __getitem__ and
    reverse (text->value) lookups via __getattr__. Applications
    using the old interface will need to be updated.
 
@@ -38,18 +40,18 @@ major update to the library - the key changes include:
    line with RFC)
 
  * Most object attributes are now typed in line with the record
-   definitions to make it harder to generate invalid packets 
+   definitions to make it harder to generate invalid packets
 
- * Support for encoding/decoding resource records in 'Zone' (BIND) 
-   file format 
+ * Support for encoding/decoding resource records in 'Zone' (BIND)
+   file format
 
  * Support for encoding/decoding packets in 'DiG' format
 
  * Server framework allowing (in most cases) custom resolvers to
-   be created by just subclassing the DNSResolver class and 
+   be created by just subclassing the DNSResolver class and
    overriding the 'resolve' method
 
- * A lot of fixes to error detection/handling which should make 
+ * A lot of fixes to error detection/handling which should make
    the library much more robust to invalid/unsupported data. The
    library should now either return a valid DNSRecord instance
    when parsing a packet or raise DNSError (tested via fuzzing)
@@ -65,15 +67,15 @@ major update to the library - the key changes include:
 Classes
 -------
 
-The key DNS packet handling classes are in dnslib.dns and map to the 
+The key DNS packet handling classes are in dnslib.dns and map to the
 standard DNS packet sections:
 
  * DNSRecord - container for DNS packet. Contains:
-    - DNSHeader 
-    - Question section containing zero or more DNSQuestion objects 
-    - Answer section containing zero or more RR objects 
-    - Authority section containing zero or more RR objects 
-    - Additional section containing zero or more RR objects 
+    - DNSHeader
+    - Question section containing zero or more DNSQuestion objects
+    - Answer section containing zero or more RR objects
+    - Authority section containing zero or more RR objects
+    - Additional section containing zero or more RR objects
  * DNS RRs (resource records) contain an RR header and an RD object)
  * Specific RD types are implemented as subclasses of RD
  * DNS labels are represented by a DNSLabel class - in most cases
@@ -202,7 +204,7 @@ format defined in RFC1035 (specifically not $INCLUDE)
 
 To create a skeleton reply to a DNS query:
 
-    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY)) 
+    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY))
     >>> a = q.reply()
     >>> a.add_answer(RR("abc.com",QTYPE.A,rdata=A("1.2.3.4"),ttl=60))
     >>> str(DNSRecord.parse(a.pack())) == str(a)
@@ -234,7 +236,7 @@ Add additional RRs:
 
 It is also possible to create a reply from a string in zone file format:
 
-    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY)) 
+    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY))
     >>> a = q.replyZone("abc.com 60 IN CNAME xxx.abc.com")
     >>> print(a)
     ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: ...
@@ -247,7 +249,7 @@ It is also possible to create a reply from a string in zone file format:
     >>> str(DNSRecord.parse(a.pack())) == str(a)
     True
 
-    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY)) 
+    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.ANY))
     >>> a = q.replyZone(textwrap.dedent(z))
     >>> print(a)
     ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: ...
@@ -262,7 +264,7 @@ It is also possible to create a reply from a string in zone file format:
 
 To send a DNSSEC request (EDNS OPT record with DO flag & header AD flag):
 
-    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.A)) 
+    >>> q = DNSRecord(q=DNSQuestion("abc.com",QTYPE.A))
     >>> q.add_ar(EDNS0(flags="do",udp_len=4096))
     >>> q.header.ad = 1
     >>> print(q)
@@ -274,20 +276,28 @@ To send a DNSSEC request (EDNS OPT record with DO flag & header AD flag):
     ;; OPT PSEUDOSECTION
     ; EDNS: version: 0, flags: do; udp: 4096
 
+Note that when using the library you should always validate the received TXID 
+
+    q = DNSRecord.question("abc.com")
+    a_pkt = q.send(address,port,tcp=args.tcp)
+    a = DNSRecord.parse(a_pkt)
+    if q.header.id != a.header.id:
+        raise DNSError('Response transaction id does not match query transaction id')
+
 The library also includes a simple framework for generating custom DNS
-resolvers in dnslib.server (see module docs). In most cases this just 
-requires implementing a custom 'resolve' method which receives a question 
+resolvers in dnslib.server (see module docs). In most cases this just
+requires implementing a custom 'resolve' method which receives a question
 object and returns a response.
 
 A number of sample resolvers are provided as examples (see CLI --help):
 
  * dnslib.fixedresolver    - Respond to all requests with fixed response
- * dnslib.zoneresolver     - Respond from Zone file 
+ * dnslib.zoneresolver     - Respond from Zone file
  * dnslib.shellresolver    - Call shell script to generate response
 
 The library includes a number of client utilities:
 
- * DiG like client library 
+ * DiG like client library
 
         # python -m dnslib.client --help
 
@@ -326,15 +336,15 @@ Changelog:
                          Misc fixes from James Mills - thanks
  *   0.9.3   2014-08-26  Workaround for argparse bug which raises AssertionError if [] is
                          present in option text (really?)
- *   0.9.4   2015-04-10  Fix to support multiple strings in TXT record 
+ *   0.9.4   2015-04-10  Fix to support multiple strings in TXT record
                          Patch provided by James Cherry (https://bitbucket.org/james_cherry) - thanks
                          NOTE: For consistency this patch changes the 'repr' output for
                                TXT records to always be quoted
  *   0.9.5   2015-10-27  Add threading & timeout handling to DNSServer
  *   0.9.6   2015-10-28  Replace strftime in RRSIG formatting to avoid possible locale issues
                          Identified by Bryan Everly - thanks
- *   0.9.7   2017-01-15  Sort out CAA/TYPE257 DiG parsing mismatch 
- *   0.9.8   2019-02-25  Force DNSKEY key to be bytes object 
+ *   0.9.7   2017-01-15  Sort out CAA/TYPE257 DiG parsing mismatch
+ *   0.9.8   2019-02-25  Force DNSKEY key to be bytes object
                          Catch Bimap __wrapped__ attr (used by inspect module in 3.7)
  *   0.9.9   2019-03-19  Add support for DNSSEC flag getters/setters (from <raul@dinosec.com> - thanks)
                          Added --dnssec flags to dnslib.client & dnslib.test_decode (sets EDNS0 DO flag)
@@ -347,6 +357,47 @@ Changelog:
                          (Thanks to <raul@dinosec.com> for help)
  *   0.9.11  2019-12-17  Encode NOTIFY Opcode (Issue #26)
  *   0.9.12  2019-12-17  Transition master repository to Github (Bitbucket shutting down hg)
+ *   0.9.13  2020-06-01  Handle truncated requests in server.py (Issue #9)
+                         Replace thred.isAlive with thread.is_alive (Deprecated in Py3.9)
+                         Merged Pull Request #4 (Extra options for intercept.py) - thanks to @nolanl
+ *   0.9.14  2020-06-09  Merged Pull Request #10 (Return doctest status via exit code)
+                         Thanks to @mgorny
+ *   0.9.15  2021-05-07  DNSServer fixes - support IPv6 (from Pull Request #21) - thanks to @mikma
+                                         - deamon threads (Pull Request #19) - thanks to @wojons
+                         Add unsupported RR types (Issue #27)
+ *   0.9.16  2021-05-07  Merge pull request #23 from Tugzrida/patch-1
+                            Add support for all RR types to NSEC type bitmap
+                         Merge pull request #17 from sunds/issue_16
+                            Issue 16: uncaught exceptions leak open sockets
+ *   0.9.18  2022-01-09  Validate TXID in client.py (Issue #30 - thanks to @daniel4x)
+ *   0.9.19  2022-01-09  Allow custom log function (logf) in  DNSLogger
+                            (Issue #31 - thanks to @DmitryFrolovTri)
+ *   0.9.20  2022-07-17  Fix DeprecationWarnings about invalid escape sequences
+                            (Pull-Request #39 - thanks to @brianmaissy)
+                         Make DNSLabel matchSuffix and stripSuffix case-insensitive
+                            (Pull-Request #37 - thanks to @NiKiZe)
+                         Add support for HTTPS RR
+                            (Pull-Request #35 - thanks to @jkl-caliber)
+                         Fix display of non-printable characters in TXT records
+                            (Issue #32 - thanks to @sbv-csis)
+                         Add --strip-aaaa option to dnslib.proxy 
+ *   0.9.21  2022-09-19  Minor clean-up / add wheels to distro
+ *   0.9.22  2022-09027  Issue #43 (0.9.21 Raises TypeError instead of DNSError when failing to parse HTTPS records)
+                         Note that we just fix the exception - there still seems to be a problem with parsing HTTPS records
+                         (Thanks to @robinlandstrom)
+ *   0.9.23  2022-10-28  Issue #43: HTTPS reads after RD end (thanks to @robinlandstrom for pull request)
+                         Issue #45: Dnslib fails to handle unknown RR types in NSEC RD type bitmap
+                            Bimap now supports a function to map unknown types which we use to
+                            dynamically map from rtype <-> TYPExxxx for unknown record types
+                            RR zone representation updated to match RFC3597
+                         Pull Request #47: Add support for DS, SSHFP, and TLSA records (thanks to @rmbolger)
+ *   0.9.24  2024-01-02  Merge multiple PRs
+                            #49 - Generate README.md (via symlink)
+                            #51 - Update Github CI checkout & setup-python actions (and remove Python 2.7 CI support)
+                                  (thanks to @SpencerIsGiddy)
+                            #54 - Support for RP records (thanks to @ryan-gang)
+                            #57 - Support for LOC records (thanks to @valentinesd)
+                         (Note that this will be the last release supporting Python 2.7 and Python <3.7)
 
 License:
 --------
@@ -356,12 +407,12 @@ BSD
 Author:
 -------
 
- *   PaulC 
+ *   PaulC
 
 Master Repository/Issues:
 -------------------------
 
- *   https://github.com/paulc/dnslib  
+ *   https://github.com/paulc/dnslib
 
  (Note: https://bitbucket.org/paulc/dnslib has been deprecated and will not be updated)
 
@@ -369,9 +420,8 @@ Master Repository/Issues:
 
 from dnslib.dns import *
 
-version = "0.9.12"
+version = "0.9.24"
 
 if __name__ == '__main__':
-    import doctest,textwrap
-    doctest.testmod(optionflags=doctest.ELLIPSIS)
-
+    import doctest,sys,textwrap
+    sys.exit(0 if doctest.testmod(optionflags=doctest.ELLIPSIS).failed == 0 else 1)
